@@ -20,24 +20,32 @@ module Control.Arrow.Transformer.Stream(
 		ArrowAddStream(..),
 	) where
 
-import Control.Monad.ST
-
-import Control.Arrow
 import Control.Arrow.Internals
 import Control.Arrow.Operations
 import Control.Arrow.Transformer
+
+import Control.Applicative
+import Control.Arrow
+import Control.Category
+import Control.Monad.ST
+import Data.Monoid
 import Data.Stream (Stream(..))
 import qualified Data.Stream as Stream
 
+import Prelude hiding (id,(.))
+
 -- | Arrows between streams.
 --
 -- /Note/: 'lift' is only a functor if '***' in the underlying arrow is.
 
 newtype StreamArrow a b c = Str (a (Stream b) (Stream c))
 
+instance Category a => Category (StreamArrow a) where
+	id = Str id
+	Str f . Str g = Str (f . g)
+
 instance Arrow a => Arrow (StreamArrow a) where
 	arr f = Str (arr (fmap f))
-	Str f >>> Str g = Str (f >>> g)
 	first (Str f) =
 		Str (arr Stream.unzip >>> first f >>> arr (uncurry Stream.zip))
 
@@ -94,6 +102,23 @@ instance ArrowPlus a => ArrowPlus (StreamArrow a) where
 instance ArrowLoop a => ArrowCircuit (StreamArrow a) where
 	delay x = Str (arr (Cons x))
 
+-- Other instances
+
+instance Arrow a => Functor (StreamArrow a b) where
+	fmap f g = g >>> arr f
+
+instance Arrow a => Applicative (StreamArrow a b) where
+	pure x = arr (const x)
+	f <*> g = f &&& g >>> arr (uncurry id)
+
+instance ArrowPlus a => Alternative (StreamArrow a b) where
+	empty = zeroArrow
+	f <|> g = f <+> g
+
+instance ArrowPlus a => Monoid (StreamArrow a b c) where
+	mempty = zeroArrow
+	mappend f g = f <+> g
+
 -- | Run a stream processor on a stream of inputs, obtaining a stream
 -- of outputs.
 --
